@@ -22,25 +22,23 @@ import tkinter as tk
 from tkinter import ttk
 import tkinter.filedialog as filedialog
 import tkinter.simpledialog as simpledialog
-import tkinter.colorchooser as colorchooser
 import tkinter.font as font
-from ttkwidgets import CheckboxTreeview
 
 import matplotlib.pyplot as plt
 from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
 from matplotlib import rcParams
+import matplotlib.gridspec
 import numpy as np
 
-import lark
-from lark import Lark, Transformer, v_args, exceptions
-
 # https://qiita.com/yniji/items/3fac25c2ffa316990d0c matplotlibで日本語を使う
 rcParams['font.family'] = 'sans-serif'
 rcParams['font.sans-serif'] = ['Hiragino Sans', 'Yu Gothic', 'Meirio', 'Takao', 'IPAexGothic', 'IPAPGothic', 'VL PGothic', 'Noto Sans CJK JP']
 
 from ._version import __version__
-from . import mapinterpleter as interp
+from . import mapinterpreter as interp
 from . import mapplot
+from . import dialog_multifields
+from . import othertrack_window
 
 # http://centerwave-callout.com/tkinter内で起きた例外をどうキャッチするか？/
 class Catcher: # tkinter内で起きた例外をキャッチする
@@ -62,146 +60,6 @@ class Catcher: # tkinter内で起きた例外をキャッチする
                 tk.messagebox.showinfo(message=e)
 
 class mainwindow(ttk.Frame):
-    class SubWindow(ttk.Frame):
-        def __init__(self, master, mainwindow):
-            self.mainwindow = mainwindow
-            self.parent = master
-            super().__init__(master, padding='3 3 3 3')
-            
-            self.mainwindow.tk.eval("""
-                ttk::style map Treeview \
-                -foreground {disabled SystemGrayText \
-                             selected SystemHighlightText} \
-                -background {disabled SystemButtonFace \
-                             selected SystemHighlight}
-            """)
-            
-            self.master.title('Other tracks')
-            self.master.columnconfigure(0, weight=1)
-            self.master.rowconfigure(0, weight=1)
-            self.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
-            self.create_widgets()
-            self.master.geometry('+1100+0')
-        def create_widgets(self):
-            self.othertrack_tree = CheckboxTreeview(self, show='tree headings', columns=['mindist', 'maxdist', 'linecolor'],selectmode='browse')
-            self.othertrack_tree.bind("<ButtonRelease>", self.click_tracklist)
-            self.othertrack_tree.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E))
-            self.othertrack_tree.column('#0', width=200)
-            self.othertrack_tree.column('mindist', width=100)
-            self.othertrack_tree.column('maxdist', width=100)
-            self.othertrack_tree.column('linecolor', width=50)
-            self.othertrack_tree.heading('#0', text='track key')
-            self.othertrack_tree.heading('mindist', text='From')
-            self.othertrack_tree.heading('maxdist', text='To')
-            self.othertrack_tree.heading('linecolor', text='Color')
-            
-            self.ottree_scrollbar = ttk.Scrollbar(self, orient=tk.VERTICAL, command=self.othertrack_tree.yview)
-            self.ottree_scrollbar.grid(column=1, row=0, sticky=(tk.N, tk.S))
-            self.othertrack_tree.configure(yscrollcommand=self.ottree_scrollbar.set)
-
-        def click_tracklist(self, event=None):
-            '''他軌道リストをクリックしたときのイベント処理
-            '''
-            columnlabel = {'#0':'Check','#1':'From', '#2':'To', '#3':'Color'}
-            if event != None:
-                if getattr(event, 'widget').identify("element", event.x, event.y) == 'text': #パラメータ部分クリックかどうか。チェックボックスだとimage
-                    clicked_column = self.othertrack_tree.identify_column(event.x)
-                    clicked_track = self.othertrack_tree.identify_row(event.y)
-                    #print(clicked_track,columnlabel[clicked_column])
-                    if clicked_column in ['#1','#2','#3'] and clicked_track != 'root':
-                        if clicked_column == '#3': # ラインカラーかどうか？
-                            inputdata = colorchooser.askcolor(color=self.mainwindow.result.othertrack_linecolor[clicked_track]['current'],title=clicked_track+' ,default: '+self.mainwindow.result.othertrack_linecolor[clicked_track]['default'])
-                            if inputdata[1] != None:
-                                self.mainwindow.result.othertrack_linecolor[clicked_track]['current'] = inputdata[1]
-                                self.othertrack_tree.tag_configure(clicked_track,foreground=self.mainwindow.result.othertrack_linecolor[clicked_track]['current'])
-                        else:
-                            if clicked_column == '#1':
-                                defaultval = min(self.mainwindow.result.othertrack.data[clicked_track], key=lambda x: x['distance'])['distance']
-                            elif clicked_column == '#2':
-                                defaultval = max(self.mainwindow.result.othertrack.data[clicked_track], key=lambda x: x['distance'])['distance']
-                            inputdata = simpledialog.askfloat(clicked_track+': Distance', columnlabel[clicked_column]+' (default: '+str(defaultval)+' m)')
-                            #print(inputdata)
-                            if inputdata != None: # 入力値に問題なければ、描画範囲を変更する
-                                if clicked_column == '#1':
-                                    self.mainwindow.result.othertrack.cp_range[clicked_track]['min'] = inputdata
-                                elif clicked_column == '#2':
-                                    self.mainwindow.result.othertrack.cp_range[clicked_track]['max'] = inputdata
-                                self.othertrack_tree.set(clicked_track,clicked_column,inputdata) # 他軌道リストの表示値変更
-            self.mainwindow.plot_all()
-        def set_ottree_value(self):
-            if self.othertrack_tree.exists('root'):
-                self.othertrack_tree.delete('root')
-            self.othertrack_tree.insert("", "end", 'root', text='root', open=True)
-            colorix = 0
-            for i in self.mainwindow.result.othertrack.data.keys():
-                self.othertrack_tree.insert("root", "end", i, text=i, values=(min(self.mainwindow.result.othertrack.data[i], key=lambda x: x['distance'])['distance'],max(self.mainwindow.result.othertrack.data[i], key=lambda x: x['distance'])['distance'], '■■■'),tags=(i,))
-                self.othertrack_tree.tag_configure(i,foreground=self.mainwindow.result.othertrack_linecolor[i]['current'])
-            #self.subwindow.othertrack_tree.see('root')
-            #self.othertrack_tree.configure(yscrollcommand=self.ottree_scrollbar.set)
-    class dialog_multifields(ttk.Frame):
-        def __init__(self, mainwindow, variable, title=None, message=None):
-            self.mainwindow = mainwindow
-            self.master = tk.Toplevel(self.mainwindow)
-            super().__init__(self.master, padding='3 3 3 3')
-            if title != None:
-                self.master.title(title)
-            else:
-                self.master.title('')
-            self.master.columnconfigure(0, weight=1)
-            self.master.rowconfigure(0, weight=1)
-            self.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
-            self.create_widgets(variable, message)
-            self.master.bind("<Return>", self.clickOk)
-            self.master.bind("<Escape>", self.clickCancel)
-            
-            self.result = False
-            
-            self.master.focus_set()
-            self.master.grab_set()
-            self.master.transient(self.mainwindow)
-            self.master.wait_window()
-        def create_widgets(self,variable,message):
-            self.variables = {}
-            self.entries = {}
-            self.labels = {}
-            
-            if message != None:
-                self.message_label = ttk.Label(self,text = message)
-                self.message_label.grid(column=0, row=0, sticky=(tk.W,tk.N))
-            
-            self.entry_frame = ttk.Frame(self, padding='3 3 3 3')
-            self.entry_frame.grid(column=0, row=1, sticky=(tk.N))
-            
-            ix=0
-            for key in variable:
-                if key['type'] == 'str':
-                    self.variables[key['name']] = tk.StringVar()
-                else:
-                    self.variables[key['name']] = tk.DoubleVar()
-                self.labels[key['name']] = ttk.Label(self.entry_frame,text = key['label'])
-                self.labels[key['name']].grid(column=0, row=ix, sticky=(tk.W))
-                self.entries[key['name']] = ttk.Entry(self.entry_frame,textvariable=self.variables[key['name']],width=7)
-                self.entries[key['name']].grid(column=1, row=ix, sticky=(tk.E))
-                self.variables[key['name']].set(key['default'])
-                ix+=1
-            
-            self.button_frame = ttk.Frame(self, padding='3 3 3 3')
-            self.button_frame.grid(column=0, row=2, sticky=(tk.E,tk.W))
-            self.button_ok = ttk.Button(self.button_frame, text="OK", command=self.clickOk)
-            self.button_ok.grid(column=0, row=0, sticky=(tk.S))
-            self.button_reset = ttk.Button(self.button_frame, text="Reset", command=self.clickreset)
-            self.button_reset.grid(column=1, row=0, sticky=(tk.S))
-            self.button_cancel = ttk.Button(self.button_frame, text="Cancel", command=self.clickCancel)
-            self.button_cancel.grid(column=2, row=0, sticky=(tk.S))
-        def clickOk(self, event=None):
-            self.result = 'OK'
-            self.master.destroy()
-        def clickreset(self, event=None):
-            self.result = 'reset'
-            self.master.destroy()
-        def clickCancel(self, event=None):
-            self.master.destroy()
-            
     def __init__(self, master, parser):
         self.dmin = None
         self.dmax = None
@@ -210,20 +68,21 @@ class mainwindow(ttk.Frame):
         
         super().__init__(master, padding='3 3 3 3')
         self.master.title('Kobushi trackviewer')
+        self.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
         self.master.columnconfigure(0, weight=1)
         self.master.rowconfigure(0, weight=1)
-        self.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
         
         master.protocol('WM_DELETE_WINDOW', self.ask_quit)
         
         self.create_widgets()
         self.create_menubar()
         self.bind_keyevent()
-        self.subwindow = self.SubWindow(tk.Toplevel(master), self)
+        self.subwindow = othertrack_window.SubWindow(tk.Toplevel(master), self)
         
         self.parser = parser
 
     def create_widgets(self):
+        # 補助情報フレーム
         self.control_frame = ttk.Frame(self, padding='3 3 3 3')
         self.control_frame.grid(column=1, row=1, sticky=(tk.S))
         
@@ -270,6 +129,7 @@ class mainwindow(ttk.Frame):
         self.prof_othert_chk = ttk.Checkbutton(self.aux_values_control, text='縦断面図他軌道',onvalue=True, offvalue=False, variable=self.prof_othert_val, command=self.plot_all)
         self.prof_othert_chk.grid(column=0, row=6, sticky=(tk.N, tk.W, tk.E))
         
+        # 描画区間フレーム
         self.distlimit_control = ttk.Frame(self.control_frame, padding='3 3 3 3', borderwidth=1, relief='ridge')
         self.distlimit_control.grid(column=0, row=2, sticky=(tk.S, tk.W, tk.E))
         self.distlimit_label =  ttk.Label(self.distlimit_control, text='描画区間', font = font_title)
@@ -287,8 +147,9 @@ class mainwindow(ttk.Frame):
         self.distlimit_label =  ttk.Label(self.dist_range_arb_frame, text='m')
         self.distlimit_label.grid(column=2, row=0, sticky=(tk.W))
         
+        # ファイルパスフレーム
         self.file_frame = ttk.Frame(self, padding='3 3 3 3')
-        self.file_frame.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
+        self.file_frame.grid(column=0, row=0, sticky=(tk.N, tk.W))
         self.open_btn = ttk.Button(self.file_frame, text="開く", command=self.open_mapfile)
         self.open_btn.grid(column=0, row=0, sticky=(tk.W))
         self.filedir_entry_val = tk.StringVar()
@@ -296,23 +157,24 @@ class mainwindow(ttk.Frame):
         self.filedir_entry.grid(column=1, row=0, sticky=(tk.W, tk.E))
         
         self.setdist_frame = ttk.Frame(self, padding='3 3 3 3')
-        self.setdist_frame.grid(column=0, row=2, sticky=(tk.W, tk.E))
+        self.setdist_frame.grid(column=0, row=2, sticky=(tk.S, tk.W, tk.E))
         
         self.setdist_entry_frame = ttk.Frame(self.setdist_frame, padding='0 0 0 0')
         self.setdist_entry_frame.grid(column=0, row=0, sticky=(tk.W, tk.E))
         self.distset_btn = ttk.Button(self.setdist_entry_frame, text="距離程セット", command=self.distset_entry, width=12)
-        self.distset_btn.grid(column=0, row=0, sticky=(tk.W, tk.E))
+        self.distset_btn.grid(column=0, row=0, sticky=(tk.W))
         self.setdist_entry_val = tk.DoubleVar()
         self.setdist_entry = ttk.Entry(self.setdist_entry_frame, width=7, textvariable=self.setdist_entry_val)
-        self.setdist_entry.grid(column=1, row=0, sticky=(tk.W, tk.E))
+        self.setdist_entry.grid(column=1, row=0, sticky=(tk.W))
         self.setdist_entry_label = ttk.Label(self.setdist_entry_frame, text='m')
-        self.setdist_entry_label.grid(column=2, row=0, sticky=(tk.E))
+        self.setdist_entry_label.grid(column=2, row=0, sticky=(tk.W))
         
-        self.distance_scale = ttk.Scale(self.setdist_frame, orient=tk.HORIZONTAL, length=500, from_=0, to=100, command=self.setdist_scale)
+        # 距離程フレーム
+        self.distance_scale = ttk.Scale(self.setdist_frame, orient=tk.HORIZONTAL, from_=0, to=100, command=self.setdist_scale)#,length=500)
         self.distance_scale.grid(column=1, row=0, sticky=(tk.W, tk.E))
         
         self.stationlist_frame = ttk.Frame(self.setdist_frame, padding='0 0 0 0')
-        self.stationlist_frame.grid(column=2, row=0, sticky=(tk.W, tk.E))
+        self.stationlist_frame.grid(column=2, row=0, sticky=(tk.E))
         self.stationlist_label = ttk.Label(self.stationlist_frame, text='駅移動', font = font_title)
         self.stationlist_label.grid(column=0, row=0, sticky=(tk.W))
         self.stationlist_val = tk.StringVar()
@@ -320,24 +182,63 @@ class mainwindow(ttk.Frame):
         self.stationlist_cb.grid(column=1, row=0, sticky=(tk.W, tk.E))
         self.stationlist_cb.bind('<<ComboboxSelected>>', self.jumptostation)
         
+        self.setdist_frame.columnconfigure(0, weight=0)
+        self.setdist_frame.columnconfigure(1, weight=1)
+        self.setdist_frame.rowconfigure(0, weight=1)
+        
+        # プロットフレーム
         self.canvas_frame = ttk.Frame(self, padding='3 3 3 3')
         self.canvas_frame.grid(column=0, row=1, sticky=(tk.N, tk.W, tk.E, tk.S))
         
-        self.fig_plane = plt.figure(figsize=(10,5))
-        self.ax_plane = self.fig_plane.add_subplot()
+        self.fig_plane = plt.figure(figsize=(9,7),tight_layout=True)
         
-        self.plane_canvas = FigureCanvasTkAgg(self.fig_plane, master=self.canvas_frame)
-        self.plane_canvas.draw()
-        self.plane_canvas.get_tk_widget().grid(row = 0, column = 0)
+        gs1 = self.fig_plane.add_gridspec(nrows=2,ncols=1,height_ratios=[22, 13])
+        gs2 = gs1[1].subgridspec(nrows=3,ncols=1,height_ratios=[3, 6, 4],hspace=0)
+        self.ax_plane = self.fig_plane.add_subplot(gs1[0])
+        self.ax_profile_s = self.fig_plane.add_subplot(gs2[0])
+        self.ax_profile_g = self.fig_plane.add_subplot(gs2[1], sharex=self.ax_profile_s)
+        self.ax_profile_r = self.fig_plane.add_subplot(gs2[2], sharex=self.ax_profile_s)
         
-        self.fig_profile, ((self.ax_profile_s, self.ax_profile_g, self.ax_profile_r)) = plt.subplots(3,1, figsize=(10,3), sharex='col', gridspec_kw={'height_ratios': [3, 6, 4]})
-        self.fig_profile.subplots_adjust(hspace=0)
-        self.ax_profile_s.tick_params(labelleft=False, left=False)
+        self.ax_profile_s.tick_params(labelleft=False, left=False, labelbottom=False, bottom=False)
+        self.ax_profile_g.tick_params(labelbottom=False, bottom=False)
         self.ax_profile_r.tick_params(labelleft=False, left=False)
         
-        self.profile_canvas = FigureCanvasTkAgg(self.fig_profile, master=self.canvas_frame)
-        self.profile_canvas.draw()
-        self.profile_canvas.get_tk_widget().grid(row = 1, column = 0)
+        self.plt_canvas_base = tk.Canvas(self.canvas_frame, bg="white", width=900, height=700)
+        self.plt_canvas_base.grid(row = 0, column = 0)
+        
+        '''
+        self.fig_xscroll = ttk.Scrollbar(self.canvas_frame, orient=tk.HORIZONTAL, command=self.plt_canvas_base.xview)
+        self.fig_xscroll.grid(row = 1, column = 0, sticky=(tk.N, tk.W, tk.E, tk.S))
+        self.fig_yscroll = ttk.Scrollbar(self.canvas_frame, orient=tk.VERTICAL, command=self.plt_canvas_base.yview)
+        self.fig_yscroll.grid(row = 0, column = 1, sticky=(tk.N, tk.W, tk.E, tk.S))
+        '''
+        def on_canvas_resize(event):
+            self.plt_canvas_base.itemconfigure(self.fig_frame_id, width=event.width, height=event.height)
+            #print(event)
+        
+        self.fig_frame = tk.Frame(self.plt_canvas_base)
+        self.fig_frame_id = self.plt_canvas_base.create_window((0, 0), window=self.fig_frame, anchor="nw")
+        self.fig_frame.columnconfigure(0, weight=1)
+        self.fig_frame.rowconfigure(0, weight=1)
+        #self.fig_frame.bind("<Configure>",lambda e: self.plt_canvas_base.configure(scrollregion=self.plt_canvas_base.bbox("all")))
+        self.plt_canvas_base.bind("<Configure>", on_canvas_resize)
+        #self.plt_canvas_base.configure(yscrollcommand=self.fig_yscroll.set, xscrollcommand=self.fig_xscroll.set)
+        
+        self.fig_canvas = FigureCanvasTkAgg(self.fig_plane, master=self.fig_frame)
+        self.fig_canvas.draw()
+        self.fig_canvas.get_tk_widget().grid(row=0, column=0, sticky='news')
+        
+        self.canvas_frame.columnconfigure(0, weight=1)
+        self.canvas_frame.columnconfigure(1, weight=1)
+        self.canvas_frame.rowconfigure(0, weight=1)
+        self.canvas_frame.rowconfigure(1, weight=1)
+        
+        # ウィンドウリサイズに対する設定
+        self.columnconfigure(0, weight=1)
+        self.columnconfigure(1, weight=0)
+        self.rowconfigure(0, weight=0)
+        self.rowconfigure(1, weight=1)
+        self.rowconfigure(2, weight=0)
     def create_menubar(self):
         self.master.option_add('*tearOff', False)
         
@@ -502,7 +403,7 @@ class mainwindow(ttk.Frame):
         if self.stationpos_val.get():
             self.mplot.stationpoint_plane(self.ax_plane,labelplot=self.stationlabel_val.get())
         
-        self.plane_canvas.draw()
+        self.fig_canvas.draw()
     def draw_profileplot(self):
         self.ax_profile_g.cla()
         self.ax_profile_r.cla()
@@ -519,7 +420,7 @@ class mainwindow(ttk.Frame):
             self.mplot.gradient_value(self.ax_profile_g,labelplot=self.gradientval_val.get())
         self.mplot.radius_value(self.ax_profile_r,labelplot=self.curveval_val.get())
         
-        self.profile_canvas.draw()
+        self.fig_canvas.draw()
     def print_debugdata(self):
         if not __debug__:
             print('own_track data')
@@ -633,7 +534,7 @@ class mainwindow(ttk.Frame):
                 np.savetxt(output_filename, output, delimiter=',',header=header,fmt='%.6f')
     def set_plotlimit(self, event=None):
         if self.result != None:
-            dialog = self.dialog_multifields(self,\
+            dialog = dialog_multifields.dialog_multifields(self,\
                                             [{'name':'min', 'type':'Double', 'label':'min (default:'+str(self.result.cp_defaultrange[0])+')', 'default':self.distrange_min},\
                                             {'name':'max', 'type':'Double', 'label':'max (default:'+str(self.result.cp_defaultrange[1])+')', 'default':self.distrange_max}],
                                             message ='Set plotlimit\n'+'map range:'+str(min(self.result.controlpoints.list_cp))+','+str(max(self.result.controlpoints.list_cp)))
@@ -654,7 +555,7 @@ class mainwindow(ttk.Frame):
             equaldist_unit = 25
             cp_arbcp_default = [round(min(list_cp),-2) - boundary_margin,round(max(list_cp),-2) + boundary_margin,equaldist_unit]
             
-            dialog = self.dialog_multifields(self,\
+            dialog = dialog_multifields.dialog_multifields(self,\
                                             [{'name':'min', 'type':'Double', 'label':'min (default: '+str(cp_arb_default[0])+')', 'default':cp_arbdistribution[0]},\
                                             {'name':'max', 'type':'Double', 'label':'max (default: '+str(cp_arb_default[1])+')', 'default':cp_arbdistribution[1]},\
                                             {'name':'interval', 'type':'Double', 'label':'interval (default: '+str(cp_arb_default[2])+')', 'default':cp_arbdistribution[2]}],
@@ -670,7 +571,7 @@ class mainwindow(ttk.Frame):
                 self.reload_map()
     def set_profYlimit(self, event=None):
         if self.result != None:
-            dialog = self.dialog_multifields(self,\
+            dialog = dialog_multifields.dialog_multifields(self,\
                                             [{'name':'min', 'type':'str', 'label':'min (default:auto)', 'default':'auto' if self.profYlim == None else str(self.profYlim[0])},\
                                             {'name':'max', 'type':'str', 'label':'max (default:auto)', 'default':'auto' if self.profYlim == None else str(self.profYlim[1])}],
                                             message ='Set profile Y limit')
@@ -691,7 +592,7 @@ class mainwindow(ttk.Frame):
         msg += 'https://www.apache.org/licenses/LICENSE-2.0'
         tk.messagebox.showinfo(message=msg)
     def customdialog_test(self, event=None):
-        dialog_obj = self.dialog_multifields(self,\
+        dialog_obj = dialog_multifields.dialog_multifields(self,\
                                         [{'name':'A', 'type':'str', 'label':'test A', 'default':'alpha'},\
                                         {'name':'B', 'type':'Double', 'label':'test B', 'default':100}],\
                                         'Test Dialog')
@@ -716,10 +617,9 @@ def main():
                 pdb.pm()
         import sys
         sys.excepthook = info
-    
-    rule = open(os.path.join(os.path.abspath(os.path.dirname(__file__)),'map-grammer.lark'), encoding='utf-8').read()
+        print('Debug mode')
     
     tk.CallWrapper = Catcher
     root = tk.Tk()
-    app = mainwindow(master=root, parser = Lark(rule, parser='lalr', maybe_placeholders=True))
+    app = mainwindow(master=root, parser = None)
     app.mainloop()
