@@ -69,7 +69,12 @@ class TrackGenerator():
         self.last_pos['radius']          = r0     if r0     != None else 0
         self.last_pos['gradient']        = gr0    if gr0    != None else 0
         self.last_pos['distance']        = dist0  if dist0  != None else min(self.list_cp)
-        self.last_pos['interpolatefunc'] = 'line'
+        self.last_pos['interpolate_func'] = 'line'
+        
+        self.radius_lastpos = {}
+        self.radius_lastpos['distance'] = self.last_pos['distance']
+        self.radius_lastpos['theta']    = self.last_pos['theta']
+        self.radius_lastpos['radius']  = self.last_pos['radius']
         
         #座標情報を格納するリスト
         self.result = [[self.last_pos['distance'],self.last_pos['x'],self.last_pos['y'],self.last_pos['z'],self.last_pos['theta'],self.last_pos['radius'],self.last_pos['gradient']]]
@@ -83,7 +88,7 @@ class TrackGenerator():
         radius_p      = TrackPointer(self.env,'radius')
         gradient_p    = TrackPointer(self.env,'gradient')
         turn_p        = TrackPointer(self.env,'turn')
-        interpolate_p = TrackPointer(self.env, 'interpolate_func')
+        interpolate_p = TrackPointer(self.env,'interpolate_func')
         
         grad_gen = tc.gradient_intermediate()
         curve_gen = tc.curve_intermediate()
@@ -94,6 +99,10 @@ class TrackGenerator():
                 raise RuntimeWarning('Numpy warning: '+str(err)+', '+str(flag)+' at '+str(dist))
             np.seterr(all='call')
             np.seterrcall(print_warning_position)
+            
+        #import pdb
+        #pdb.set_trace()
+        
         for dist in self.list_cp:
             # curve.setfunction に対する処理
             while (interpolate_p.onNextpoint(dist)): #注目している要素区間の終端に到達？
@@ -106,9 +115,24 @@ class TrackGenerator():
             while (radius_p.overNextpoint(dist)): #注目している要素区間の終端を超えたか？
                 if(radius_p.seekoriginofcontinuous(radius_p.pointer['next']) != None):
                     self.last_pos['radius'] = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
+                    self.radius_lastpos['radius']  = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
+                    self.radius_lastpos['distance'] = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['distance']
+                    self.radius_lastpos['theta']    = self.last_pos['theta']
                 radius_p.seeknext()
             if(radius_p.pointer['last'] == None): # 最初のcurve要素に到達していない場合
-                if(self.last_pos['radius'] == 0):
+                if(radius_p.pointer['next'] == None): # curve要素が存在しないマップの場合
+                    if(self.last_pos['radius'] == 0):
+                        [x, y] =curve_gen.straight(self.cp_max - self.cp_min,\
+                                                    self.last_pos['theta'],\
+                                                    dist - self.last_pos['distance'])
+                        tau = 0
+                        radius = self.last_pos['radius']
+                    else:
+                        [x, y], tau =curve_gen.circular_curve(self.cp_max - self.cp_min,\
+                                                                self.last_pos['theta'],\
+                                                                dist - self.last_pos['distance'])
+                        radius = self.last_pos['radius']
+                elif(self.last_pos['radius'] == 0):
                     [x, y] =curve_gen.straight(self.data_ownt[radius_p.pointer['next']]['distance'] - self.cp_min,\
                                                 self.last_pos['theta'],\
                                                 dist - self.last_pos['distance'])
@@ -146,13 +170,31 @@ class TrackGenerator():
                     radius = self.last_pos['radius']
                 else: # 曲線半径が変化する場合
                     if(self.data_ownt[radius_p.pointer['next']]['flag'] == 'i' or self.data_ownt[radius_p.pointer['last']]['flag'] == 'bt'): # interpolateフラグがある
-                        if(self.last_pos['radius'] != self.data_ownt[radius_p.pointer['next']]['value']): # 注目区間前後で異なる曲線半径を取るなら緩和曲線を出力
-                            [x, y], tau, radius = curve_gen.transition_curve(self.data_ownt[radius_p.pointer['next']]['distance'] - self.last_pos['distance'],\
+                        if(self.radius_lastpos['radius'] != self.data_ownt[radius_p.pointer['next']]['value']): # 注目区間前後で異なる曲線半径を取るなら緩和曲線を出力
+                            '''
+                            [x, y], tau, radius = curve_gen.transition_curve(self.data_ownt[radius_p.pointer['next']]['distance'] - self.data_ownt[radius_p.pointer['last']]['distance'],\
                                                                                 self.last_pos['radius'],\
                                                                                 self.data_ownt[radius_p.pointer['next']]['value'],\
                                                                                 self.last_pos['theta'],\
-                                                                                self.last_pos['interpolatefunc'],\
-                                                                                dist - self.last_pos['distance'])
+                                                                                self.last_pos['interpolate_func'],\
+                                                                                dist - self.data_ownt[radius_p.pointer['last']]['distance'])
+                            '''
+                            pos_last            = curve_gen.transition_curve(self.data_ownt[radius_p.pointer['next']]['distance'] - self.data_ownt[radius_p.pointer['last']]['distance'],\
+                                                self.radius_lastpos['radius'],\
+                                                self.data_ownt[radius_p.pointer['next']]['value'],\
+                                                self.radius_lastpos['theta'],\
+                                                self.last_pos['interpolate_func'],\
+                                                self.last_pos['distance'] - self.data_ownt[radius_p.pointer['last']]['distance'])
+                            [x, y], tau, radius = curve_gen.transition_curve(self.data_ownt[radius_p.pointer['next']]['distance'] - self.data_ownt[radius_p.pointer['last']]['distance'],\
+                                                self.radius_lastpos['radius'],\
+                                                self.data_ownt[radius_p.pointer['next']]['value'],\
+                                                self.radius_lastpos['theta'],\
+                                                self.last_pos['interpolate_func'],\
+                                                dist - self.data_ownt[radius_p.pointer['last']]['distance'])
+                                                
+                            x -= pos_last[0][0]
+                            y -= pos_last[0][1]
+                            tau -= pos_last[1]
                         elif(self.data_ownt[radius_p.pointer['next']]['value'] != 0): # 曲線半径が変化せず、!=0の場合は円軌道を出力
                             [x, y], tau = curve_gen.circular_curve(self.data_ownt[radius_p.pointer['next']]['distance'] - self.last_pos['distance'],\
                                                                     self.last_pos['radius'],\
@@ -186,7 +228,8 @@ class TrackGenerator():
             # gradientに対する処理
             while(gradient_p.overNextpoint(dist)): #注目している要素区間の終端を超えたか？
                 if(gradient_p.seekoriginofcontinuous(gradient_p.pointer['next']) != None):
-                    self.last_pos['gradient'] = self.data_ownt[gradient_p.seekoriginofcontinuous(gradient_p.pointer['next'])]['value']
+                    self.last_pos['gradient']  = self.data_ownt[gradient_p.seekoriginofcontinuous(gradient_p.pointer['next'])]['value']
+                    self.last_pos['dist_grad'] = self.data_ownt[gradient_p.seekoriginofcontinuous(gradient_p.pointer['next'])]['distance']
                 gradient_p.seeknext()
             if(gradient_p.pointer['last'] == None): #最初の勾配要素に到達していない
                 if(gradient_p.pointer['next'] == None): # 勾配が存在しないmapの場合の処理
@@ -236,7 +279,7 @@ class TrackGenerator():
             self.last_pos['gradient'] = gradient
             self.last_pos['distance'] = dist
             # 座標リストに追加
-            self.result.append([self.last_pos['distance'],\
+            self.result.append([dist,\
                 self.last_pos['x'],\
                 self.last_pos['y'],\
                 self.last_pos['z'],\
